@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Fanout, Inc.
+ * Copyright (C) 2020-2021 Fanout, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,14 +19,19 @@ pub mod arena;
 pub mod buffer;
 pub mod channel;
 pub mod connection;
+pub mod event;
+pub mod executor;
 pub mod future;
 pub mod http1;
 pub mod list;
 pub mod listener;
+pub mod reactor;
 pub mod server;
+pub mod shuffle;
 pub mod timer;
 pub mod tls;
 pub mod tnetstring;
+pub mod waker;
 pub mod websocket;
 pub mod zhttppacket;
 pub mod zhttpsocket;
